@@ -1,16 +1,12 @@
 import json
-import pickle
-import re
 import os
-from dataclasses import asdict
 
+from requests import Response, Session
 from requests.adapters import HTTPAdapter, Retry
-from requests.cookies import RequestsCookieJar
-from requests import Session
-
-from .auth_token import AuthToken
-from .exc import GarthException
 
+from . import sso
+from .auth_tokens import OAuth1Token, OAuth2Token
+from .utils import asdict
 
 USER_AGENT = {
     "User-Agent": (
@@ -22,16 +18,18 @@ USER_AGENT = {
 
 class Client:
     sess: Session
+    last_resp: Response
     domain: str = "garmin.com"
-    auth_token: AuthToken | None = None
-    timeout: int = 15
+    oauth1_token: OAuth1Token | None = None
+    oauth2_token: OAuth2Token | None = None
+    timeout: int = 10
     retries: int = 3
-    status_forcelist: tuple[int, ...] = (429, 500, 502, 503, 504)
+    status_forcelist: tuple[int, ...] = (408, 429, 500, 502, 503, 504)
     backoff_factor: float = 0.5
     _username: str | None = None
 
-    def __init__(self, **kwargs):
-        self.sess = Session()
+    def __init__(self, session: Session | None = None, **kwargs):
+        self.sess = session if session else Session()
         self.sess.headers.update(USER_AGENT)
         self.configure(
             timeout=self.timeout,
@@ -44,9 +42,9 @@ class Client:
     def configure(
         self,
         /,
-        auth_token: AuthToken | None = None,
-        cookies: RequestsCookieJar | None = None,
-        domain: str | None = None,  # Set to "garmin.cn" for China
+        oauth1_token: OAuth1Token | None = None,
+        oauth2_token: OAuth2Token | None = None,
+        domain: str | None = None,
         proxies: dict | None = None,
         ssl_verify: bool | None = None,
         timeout: int | None = None,
@@ -54,13 +52,13 @@ class Client:
         status_forcelist: tuple[int, ...] | None = None,
         backoff_factor: float | None = None,
     ):
-        if auth_token:
-            self.auth_token = auth_token
-        if cookies:
-            self.sess.cookies = cookies
+        if oauth1_token is not None:
+            self.oauth1_token = oauth1_token
+        if oauth2_token is not None:
+            self.oauth2_token = oauth2_token
         if domain:
             self.domain = domain
-        if proxies:
+        if proxies is not None:
             self.sess.proxies.update(proxies)
         if ssl_verify is not None:
             self.sess.verify = ssl_verify
@@ -82,14 +80,11 @@ class Client:
         self.sess.mount("https://", adapter)
 
     @property
-    def username(self) -> str:
-        if self._username:
-            return self._username
-        resp = self.get("connect", "/modern")
-        m = re.search(r'userName":"(.+?)"', resp.text)
-        if not m:
-            raise GarthException("Could not find username")
-        self._username = m.group(1)
+    def username(self) -> str | None:
+        if not self._username:
+            self._username = self.connectapi(
+                "/userprofile-service/socialProfile"
+            )["userName"]
         return self._username
 
     def request(
@@ -99,24 +94,26 @@ class Client:
         path: str,
         /,
         api: bool = False,
+        referrer: str | bool = False,
         headers: dict = {},
         **kwargs,
     ):
         url = f"https://{subdomain}.{self.domain}{path}"
+        if referrer is True and self.last_resp:
+            headers["referer"] = self.last_resp.url
+        elif referrer:
+            headers["referer"] = referrer
         if api:
-            if self.auth_token and self.auth_token.expired:
-                self.auth_token.refresh(client=self)
-            headers["Authorization"] = str(self.auth_token)
-            headers["di-backend"] = f"connectapi.{self.domain}"
-        resp = self.sess.request(
+            headers["Authorization"] = str(self.oauth2_token)
+        self.last_resp = self.sess.request(
             method,
             url,
             headers=headers,
             timeout=self.timeout,
             **kwargs,
         )
-        resp.raise_for_status()
-        return resp
+        self.last_resp.raise_for_status()
+        return self.last_resp
 
     def get(self, *args, **kwargs):
         return self.request("GET", *args, **kwargs)
@@ -124,29 +121,34 @@ class Client:
     def post(self, *args, **kwargs):
         return self.request("POST", *args, **kwargs)
 
-    def login(self, *args, clear_cookies: bool = False):
-        if clear_cookies:
-            self.cookies = RequestsCookieJar()
-        token = AuthToken.login(*args, client=self)
-        self.auth_token = token
+    def login(self, *args):
+        self.oauth1_token, self.oauth2_token = sso.login(*args, client=self)
 
     def connectapi(self, path: str, **kwargs):
-        return self.get("connect", path, api=True, **kwargs).json()
+        resp = self.get("connectapi", path, api=True, **kwargs)
+        if resp.status_code == 204:
+            rv = None
+        else:
+            rv = resp.json()
+        return rv
 
-    def save_session(self, dir_path: str):
+    def dump(self, dir_path: str):
         dir_path = os.path.expanduser(dir_path)
-        with open(os.path.join(dir_path, "cookies.pickle"), "wb") as f:
-            pickle.dump(self.sess.cookies, f)
-        with open(os.path.join(dir_path, "auth_token.json"), "w") as f:
-            json.dump(asdict(self.auth_token) if self.auth_token else {}, f)
+        os.makedirs(dir_path, exist_ok=True)
+        with open(os.path.join(dir_path, "oauth1_token.json"), "w") as f:
+            if self.oauth1_token:
+                json.dump(asdict(self.oauth1_token), f, indent=4)
+        with open(os.path.join(dir_path, "oauth2_token.json"), "w") as f:
+            if self.oauth2_token:
+                json.dump(asdict(self.oauth2_token), f, indent=4)
 
-    def resume_session(self, dir_path: str):
+    def load(self, dir_path: str):
         dir_path = os.path.expanduser(dir_path)
-        with open(os.path.join(dir_path, "cookies.pickle"), "rb") as f:
-            cookies = pickle.load(f)
-        with open(os.path.join(dir_path, "auth_token.json")) as f:
-            auth_token = AuthToken(**json.load(f))
-        self.configure(auth_token=auth_token, cookies=cookies)
+        with open(os.path.join(dir_path, "oauth1_token.json")) as f:
+            oauth1 = OAuth1Token(**json.load(f))
+        with open(os.path.join(dir_path, "oauth2_token.json")) as f:
+            oauth2 = OAuth2Token(**json.load(f))
+        self.configure(oauth1_token=oauth1, oauth2_token=oauth2)
 
 
 client = Client()
